h1 {
    font-family: Arial, Helvetica, sans-serif;
    margin: 20px 0px;
    font-size: 64px;
}
#target {
    font-family: Arial, Helvetica, sans-serif;
    border-collapse: collapse;
}
#target td, #target th {
    border: 1px solid #c3c3c3;
    padding: 8px;
}
#target tr:nth-child(even){
    background-color: #f2f2f2;
}
#target tr:hover {
    background-color: #ddd;
}
#target th {
  padding-top: 12px;
  padding-bottom: 12px;
  text-align: left;
  background-color: #2E6F6C;
  color: white;
}
#title_header {
    width: 100%;
}